#!/bin/bash
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#*                                                                           *
#*                  This file is part of the program and library             *
#*         SCIP --- Solving Constraint Integer Programs                      *
#*                                                                           *
#*    Copyright (C) 2002-2009 Konrad-Zuse-Zentrum                            *
#*                            fuer Informationstechnik Berlin                *
#*                                                                           *
#*  SCIP is distributed under the terms of the ZIB Academic License.         *
#*                                                                           *
#*  You should have received a copy of the ZIB Academic License              *
#*  along with SCIP; see the file COPYING. If not email to scip@zib.de.      *
#*                                                                           *
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

TSTNAME=$1
CPLEXBIN=$2
SETNAME=$3
BINID=$4
BINNAME=$CPLEXBIN.$4
TIMELIMIT=$5
NODELIMIT=$6
MEMLIMIT=$7
FEASTOL=$8
MIPGAP=$9
CONTINUE=${10}
LOCK=${11}
MODEL=${12}
ZIMPLPATH=${13}
ZIMPLBIN=zimpl

if test ! -e results
then
    mkdir results
fi

OUTFILE=results/check.$MODEL.$TSTNAME.$CPLEXBIN.out
ERRFILE=results/check.$MODEL.$TSTNAME.$CPLEXBIN.err
RESFILE=results/check.$MODEL.$TSTNAME.$CPLEXBIN.res
TMPFILE=results/check.$MODEL.$TSTNAME.$CPLEXBIN.tmp

MVORCP=mv

DATEINT=`date +"%s"`
if test -e $OUTFILE
then
    $MVORCP $OUTFILE $OUTFILE.old-$DATEINT
fi
if test -e $ERRFILE
then
    $MVORCP $ERRFILE $ERRFILE.old-$DATEINT
fi

uname -a >>$OUTFILE
uname -a >>$ERRFILE
date >>$OUTFILE
date >>$ERRFILE
echo @00 $MODEL >>$OUTFILE

# we add 10% to the hard time limit and additional 10 seconds in case of small time limits
HARDTIMELIMIT=`expr \`expr $TIMELIMIT + 10\` + \`expr $TIMELIMIT / 10\``

# we add 10% to the hard memory limit and additional 100mb to the hard memory limit
HARDMEMLIMIT=`expr \`expr $MEMLIMIT + 100\` + \`expr $MEMLIMIT / 10\``
HARDMEMLIMIT=`expr $HARDMEMLIMIT \* 1024`

echo "hard time limit: $HARDTIMELIMIT s" >>$OUTFILE
echo "hard mem limit: $HARDMEMLIMIT k" >>$OUTFILE

for i in `cat $TSTNAME.test`
do
    if test "$LASTPROB" = ""
    then
	LASTPROB=""
	if test -f ../$i
	then

	    # cplex stuff
	    rm -f $SETFILE
	    echo @01 $i ===========
	    echo @01 $i ===========                 >> $ERRFILE

	    touch $MODLE.lp

            # zimpl stuff 
	    bash -c "ulimit -t $HARDTIMELIMIT; ulimit -v $HARDMEMLIMIT; ulimit -f 1000000; $ZIMPLBIN -v0 -DDATAFILE=../$i ../$ZIMPLPATH/$MODEL.zpl" 2>>$OUTFILE

	    if test $SETNAME != "default"
	    then
#	        echo read $SETTINGS                  > $TMPFILE
		cp $SETTINGS $TMPFILE
	    else
		echo ""                              > $TMPFILE
	    fi
	    if test $FEASTOL != "default"
	    then
		echo set simplex tolerances feas $FEASTOL    >> $TMPFILE
		echo set mip tolerances integrality $FEASTOL >> $TMPFILE
	    fi
	    echo set timelimit $TIMELIMIT           >> $TMPFILE
	    echo set clocktype 0                    >> $TMPFILE
	    echo set mip display 3                  >> $TMPFILE
	    echo set mip interval 10000             >> $TMPFILE
	    if test $MIPGAP != "default"
	    then
		echo set mip tolerances mipgap $MIPGAP >> $TMPFILE
	    fi
	    echo set mip limits nodes $NODELIMIT    >> $TMPFILE
	    echo set mip limits treememory $MEMLIMIT >> $TMPFILE
	    echo set threads 1                      >> $TMPFILE
	    echo read $MODEL.lp                     >> $TMPFILE
	    echo display problem stats              >> $TMPFILE
	    echo optimize                           >> $TMPFILE
	    echo quit                               >> $TMPFILE
	    echo -----------------------------
	    date
	    echo -----------------------------
	    bash -c "ulimit -t $HARDTIMELIMIT; ulimit -v $HARDMEMLIMIT; ulimit -f 1000000; $CPLEXBIN < $TMPFILE" 2>>$ERRFILE
	    echo -----------------------------
	    date
	    echo -----------------------------
	    echo =ready=
	    rm -f $MODLE.lp
	else
	    echo @02 FILE NOT FOUND: $i ===========
	    echo @02 FILE NOT FOUND: $i =========== >>$ERRFILE
	fi
    else
	echo skipping $i
	if test "$LASTPROB" = "$i"
	then
	    LASTPROB=""
        fi
    fi
done | tee -a $OUTFILE

rm -f $TMPFILE

date >>$OUTFILE
date >>$ERRFILE

./evalcheck_cplex.sh $OUTFILE
