#!/bin/bash
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#*                                                                           *
#*                  This file is part of the program and library             *
#*         SCIP --- Solving Constraint Integer Programs                      *
#*                                                                           *
#*    Copyright (C) 2002-2009 Konrad-Zuse-Zentrum                            *
#*                            fuer Informationstechnik Berlin                *
#*                                                                           *
#*  SCIP is distributed under the terms of the ZIB Academic License.         *
#*                                                                           *
#*  You should have received a copy of the ZIB Academic License              *
#*  along with SCIP; see the file COPYING. If not email to scip@zib.de.      *
#*                                                                           *
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

TSTNAME=$1
SCIPBIN=$2
SETNAME=$3
BINID=$4
BINNAME=$SCIPBIN.$4
TIMELIMIT=$5
NODELIMIT=$6
MEMLIMIT=$7
FEASTOL=$8
DISPFREQ=$9
MODEL=${12}
ZIMPLPATH=${13}

SETDIR=../settings

if test ! -e results
then
    mkdir results
fi

OUTFILE=results/check.$MODEL.$TSTNAME.$SCIPBIN.out
ERRFILE=results/check.$MODEL.$TSTNAME.$SCIPBIN.err
RESFILE=results/check.$MODEL.$TSTNAME.$SCIPBIN.res
TMPFILE=results/check.$MODEL.$TSTNAME.$SCIPBIN.tmp

SETTINGS=$SETDIR/$SETNAME.set
MVORCP=mv

DATEINT=`date +"%s"`
if test -e $OUTFILE
then
    $MVORCP $OUTFILE $OUTFILE.old-$DATEINT
fi
if test -e $ERRFILE
then
    $MVORCP $ERRFILE $ERRFILE.old-$DATEINT
fi

uname -a >>$OUTFILE
uname -a >>$ERRFILE
date >>$OUTFILE
date >>$ERRFILE
echo @00 $MODEL >>$OUTFILE

HARDTIMELIMIT=`echo "($TIMELIMIT*1.1)+10" | bc`
HARDMEMLIMIT=`echo "($MEMLIMIT*1.1+10)*1024" | bc`

echo "hard time limit: $HARDTIMELIMIT s" >>$OUTFILE
echo "hard mem limit: $HARDMEMLIMIT k" >>$OUTFILE

for i in `cat $TSTNAME.test` DONE
do
    if test "$i" == "DONE"
    then
	break
    fi

    if test "$LASTPROB" == ""
    then
	LASTPROB=""
	if test -f ../$i
	then
	    echo @01 $i ===========
	    echo @01 $i ===========                >> $ERRFILE
	    echo > $TMPFILE
	    if test $SETTINGS != "default"
	    then
		echo set load $SETTINGS                >>  $TMPFILE
	    fi
	    if test $FEASTOL != "default"
	    then
		echo set numerics feastol $FEASTOL    >> $TMPFILE
	    fi
	    echo set limits time $TIMELIMIT         >> $TMPFILE
	    echo set limits nodes $NODELIMIT        >> $TMPFILE
	    echo set limits memory $MEMLIMIT        >> $TMPFILE
	    echo set timing clocktype 1             >> $TMPFILE
	    echo set display verblevel 4            >> $TMPFILE
	    echo set display freq $DISPFREQ         >> $TMPFILE
	    echo set memory savefac 1.0             >> $TMPFILE # avoid switching to dfs - better abort with memory error
	    echo set save $SETFILE                  >> $TMPFILE
	    echo set reading zplreader changedir 0  >> $TMPFILE
	    echo set reading zplreader parameter -DDATAFILE=../$i >> $TMPFILE
	    echo read ../$ZIMPLPATH/$MODEL.zpl      >> $TMPFILE
	    echo optimize                           >> $TMPFILE
	    echo display statistics                 >> $TMPFILE
#	    echo display solution                   >> $TMPFILE
	    echo checksol                           >> $TMPFILE
	    echo quit                               >> $TMPFILE

#	    waitcplex.sh # ??????????????????

	    echo -----------------------------
	    date
	    date >>$ERRFILE
	    echo -----------------------------
	    date +"@03 %s"
	    tcsh -c "limit cputime $HARDTIMELIMIT s; limit memoryuse $HARDMEMLIMIT k; limit filesize 200 M; $SCIPBIN < $TMPFILE" 2>>$ERRFILE
	    date +"@04 %s"
	    echo -----------------------------
	    date
	    date >>$ERRFILE
	    echo -----------------------------
	    echo
	    echo =ready=
	else
	    echo @02 FILE NOT FOUND: $i ===========
	    echo @02 FILE NOT FOUND: $i =========== >>$ERRFILE
	fi
    else
	echo skipping $i
	if test "$LASTPROB" == "$i"
	then
	    LASTPROB=""
        fi
    fi
done | tee -a $OUTFILE

rm -f $TMPFILE

date >>$OUTFILE
date >>$ERRFILE

./evalcheck.sh $OUTFILE
