####################
#
# Set Covering Formulation
#
####################

####################
#
# data parsing 
#
####################

# problem name  
param name := read DATAFILE as "1s" use 1;
do print "name = ", name	;

# capacity
param capacity := read DATAFILE as "1n" use 1 skip 1;
do print "capacity = ", capacity;

# number of items
param nitems := read DATAFILE as "2n" use 1 skip 1;
do print "number of items = ", nitems;

# index set for the items
set I := {1..nitems};

# get item sizes
set sizes[<i> in I] := {read DATAFILE as "<1n>" skip 1+i use 1};
do forall <i> in I do check card(sizes[i]) == 1;

# compute maximum number of items in any feasible packing
param maxnumber := floor(capacity / min <i> in I : ord(sizes[i],1,1));
do print "maximum number of items in packing = ", maxnumber;
	
# compute all subsets of items (packings) which have at most "maxnumber" of elements
set itemSets[] := subsets(I,1,maxnumber);
set tmpP  := indexset(itemSets);

# index set for feasible packings 
set P := {<p> in tmpP with sum <i> in itemSets[p] : ord(sizes[i],1,1) <= capacity};
do print "number of feasible packings =", card(P);

####################
#
# decision variables
#
####################

# packing variables if packing is used
var x[P] binary;

####################
#
# objective function
#
####################

# minimize number of used bins
minimize usedbins:
	 sum <p> in P : x[p];

####################
#
# constraints
#
####################

# each item has to part of exactly one packing
subto assignment:
forall <i> in I do
	sum <p> in P with card({i} inter itemSets[p]) !=  0 : x[p] == 1;

